NAME = __package__
DESCRIPTION = "Industrial-strength dynamic DNS client"
VERSION = "1.0.0"
AUTHOR = "Mike Conigliaro"
CONTACT = "http://conigliaro.org"
COPYRIGHT = f"""Copyright (c) 2020 {AUTHOR}"""
